@@ -77,7 +77,7 @@ goana.MArrayLM <- function(de, coef = ncol(de), geneid = rownames(de), FDR = 0.0
 goana.default <- function(de, universe = NULL, species = "Hs", prior.prob = NULL, covariate=NULL, plot=FALSE, ...)
 #	Gene ontology analysis of DE genes
 #	Gordon Smyth and Yifang Hu
-#	Created 20 June 2014.  Last modified 27 March 2015.
+#	Created 20 June 2014.  Last modified 23 June 2016.
 {
 #	Ensure de is a list
 	if(!is.list(de)) de <- list(DE = de)
@@ -97,9 +97,6 @@ goana.default <- function(de, universe = NULL, species = "Hs", prior.prob = NULL
 	NAME[i] <- paste0("DE",i)
 	names(de) <- makeUnique(NAME)
 
-#	Select species
-	species <- match.arg(species, c("Hs", "Mm", "Rn", "Dm"))
-
 #	Fit trend in DE with respect to the covariate, combining all de lists
 	if(!is.null(covariate)) {
 		covariate <- as.numeric(covariate)
@@ -112,17 +109,27 @@ goana.default <- function(de, universe = NULL, species = "Hs", prior.prob = NULL
 		if(plot) barcodeplot(covariate, index=(isDE==1), worm=TRUE, span.worm=span)
 	}
 
-#	Load package of GO terms
-	if(!suppressPackageStartupMessages(require("GO.db", character.only = TRUE))) stop("GO.db package not available")
+#	Get access to package of GO terms
+	suppressPackageStartupMessages(OK <- requireNamespace("GO.db",quietly=TRUE))
+	if(!OK) stop("GO.db package required but not installed (or can't be loaded)")
 
-#	Load species annotation package
-	DB <- paste("org", species, "eg", "db", sep = ".")
-	if(!suppressPackageStartupMessages(require(DB, character.only = TRUE))) stop(DB,"package not available")
+#	Get access to required annotation functions
+	suppressPackageStartupMessages(OK <- requireNamespace("AnnotationDbi",quietly=TRUE))
+	if(!OK) stop("AnnotationDbi package required but not installed (or can't be loaded)")
 
-#	Get gene-GOterm mappings, and remove duplicate entries
-	GO2ALLEGS <- paste("org", species, "egGO2ALLEGS", sep = ".")
+#	Load appropriate organism package
+	orgPkg <- paste0("org.",species,".eg.db")
+	suppressPackageStartupMessages(OK <- requireNamespace(orgPkg,quietly=TRUE))
+	if(!OK) stop(orgPkg," package required but not not installed (or can't be loaded)")
+
+#	Get GO to Entrez Gene mappings
+	obj <- paste0("org.",species,".egGO2ALLEGS")
+	egGO2ALLEGS <- tryCatch(getFromNamespace(obj,orgPkg), error=function(e) FALSE)
+	if(is.logical(egGO2ALLEGS)) stop("Can't find gene ontology mappings in package ",orgPkg)
+
+#	Convert gene-GOterm mappings to data.frame and remove duplicate entries
 	if(is.null(universe)) {
-		EG.GO <- AnnotationDbi::toTable(get(GO2ALLEGS))
+		EG.GO <- AnnotationDbi::toTable(egGO2ALLEGS)
 		d <- duplicated(EG.GO[,c("gene_id", "go_id", "Ontology")])
 		EG.GO <- EG.GO[!d, ]
 		universe <- unique(EG.GO$gene_id)
@@ -140,13 +147,12 @@ goana.default <- function(de, universe = NULL, species = "Hs", prior.prob = NULL
 		}
 		universe <- universe[!dup]
 
-		GO2ALLEGS <- get(GO2ALLEGS)
-		m <- match(AnnotationDbi::Lkeys(GO2ALLEGS),universe,0L)
+		m <- match(AnnotationDbi::Lkeys(egGO2ALLEGS),universe,0L)
 		universe <- universe[m]
 		if(!is.null(prior.prob)) prior.prob <- prior.prob[m]
 
-		AnnotationDbi::Lkeys(GO2ALLEGS) <- universe
-		EG.GO <- AnnotationDbi::toTable(GO2ALLEGS)
+		AnnotationDbi::Lkeys(egGO2ALLEGS) <- universe
+		EG.GO <- AnnotationDbi::toTable(egGO2ALLEGS)
 		d <- duplicated(EG.GO[,c("gene_id", "go_id", "Ontology")])
 		EG.GO <- EG.GO[!d, ]
 	}
@@ -185,7 +191,7 @@ goana.default <- function(de, universe = NULL, species = "Hs", prior.prob = NULL
 		W <- AVE.PW*(Total-S[,"N"])/(PW.ALL-S[,"N"]*AVE.PW)
 
 #		Wallenius' noncentral hypergeometric test
-		if(!requireNamespace("BiasedUrn",quietly=TRUE)) stop("BiasedUrn package required but is not available")
+		if(!requireNamespace("BiasedUrn",quietly=TRUE)) stop("BiasedUrn package required but is not installed (or can't be loaded)")
 		for(j in 1:nsets) for(i in 1:nrow(S)) 
 			P[i,j] <- BiasedUrn::pWNCHypergeo(S[i,1+j], S[i,"N"], Total-S[i,"N"], TotalDE[[j]], W[i],lower.tail=FALSE) + BiasedUrn::dWNCHypergeo(S[i,1+j], S[i,"N"], Total-S[i,"N"], TotalDE[[j]], W[i])
 		S <- S[,-ncol(S)]
@@ -213,7 +219,7 @@ goana.default <- function(de, universe = NULL, species = "Hs", prior.prob = NULL
 topGO <- function(results, ontology = c("BP", "CC", "MF"), sort = NULL, number = 20L, truncate.term=NULL)
 #	Extract top GO terms from goana output 
 #	Gordon Smyth and Yifang Hu
-#	Created 20 June 2014. Last modified 22 April 2015.
+#	Created 20 June 2014. Last modified 23 June 2016.
 {
 #	Check results
 	if(!is.data.frame(results)) stop("results should be a data.frame.")
@@ -251,10 +257,11 @@ topGO <- function(results, ontology = c("BP", "CC", "MF"), sort = NULL, number =
 #	Sort by minimum p-value for specified gene lists
 	P.col <- 3L+nsets+isort
 	if(length(P.col)==1L) {
-		o <- order(results[,P.col])
+		P <- results[,P.col]
 	} else {
-		o <- order(do.call("pmin",as.data.frame(results[,P.col,drop=FALSE])))
+		P <- do.call("pmin",as.data.frame(results[,P.col,drop=FALSE]))
 	}
+	o <- order(P,results$N,results$Term)
 	tab <- results[o[1L:number],,drop=FALSE]
 
 #	Truncate Term column for readability
