@@ -74,23 +74,23 @@ kegga.MArrayLM <- function(de, coef = ncol(de), geneid = rownames(de), FDR = 0.0
 		kegga(de=DEGenes, universe = universe, ...)
 }
 
-kegga.default <- function(de, universe = NULL, species = "Hs", prior.prob = NULL, covariate=NULL, plot=FALSE, ...)
+kegga.default <- function(de, universe=NULL, species="Hs", species.KEGG=NULL, convert=FALSE, gene.pathway=NULL, pathway.names = NULL, prior.prob=NULL, covariate=NULL, plot=FALSE, ...)
 #	KEGG (Kyoto Encyclopedia of Genes and Genomes) pathway analysis of DE genes
 #	Gordon Smyth and Yifang Hu
-#	Created 18 May 2015.  Modified 4 August 2015.
+#	Created 18 May 2015.  Modified 16 Nov 2016.
 {
 #	Ensure de is a list
 	if(!is.list(de)) de <- list(DE = de)
+	nsets <- length(de)
 
 #	Stop if components of de are not vectors
 	if(!all(vapply(de,is.vector,TRUE))) stop("components of de should be vectors")
 
 #	Ensure gene IDs are of character mode
-	de <- lapply(de, as.character)
+	for (s in 1:nsets) de[[s]] <- unique(as.character(de[[s]]))
 	if(!is.null(universe)) universe <- as.character(universe)
 
 #	Ensure all gene sets have unique names
-	nsets <- length(de)
 	names(de) <- trimWhiteSpace(names(de))
 	NAME <- names(de)
 	i <- which(NAME == "" | is.na(NAME))
@@ -98,125 +98,140 @@ kegga.default <- function(de, universe = NULL, species = "Hs", prior.prob = NULL
 	names(de) <- makeUnique(NAME)
 
 #	Select species
-	species <- match.arg(species, c("Hs", "Mm", "Rn", "Dm"))
+	if(is.null(species.KEGG)) {
+		species <- match.arg(species, c("Ag", "At", "Bt", "Ce", "Dm", "Dr", "EcK12", "EcSakai", "Gg", "Hs", "Mm", "Mmu", "Pf", "Pt", "Rn", "Ss", "Xl"))
+#		Convert from Bioconductor to KEGG species codes
+		species.KEGG <- switch(species, "Ag"="aga", "At"="ath", "Bt"="bta", "Ce"="cel", "Cf"="cfa", "Dm"="dme", "Dr"="dre", "EcK12"="eco", "EcSakai"="ecs", "Gg"="gga", "Hs"="hsa", "Mm"="mmu", "Mmu"="mcc", "Pf"="pfa", "Pt"="ptr", "Rn"="rno", "Ss"="ssc", "Xl"="xla")
+	}
+
+#	prior.prob and covariate must have same length as universe
+	if(is.null(universe)) {
+		if(!is.null(prior.prob)) stop("prior.prob ignored unless universe is specified.")
+		if(!is.null(covariate)) stop("covariate ignored unless universe is specified.")
+		prior.prob <- covariate <- NULL
+	} else {
+		lu <- length(universe)
+		if(!lu) stop("No genes in universe")
+		if(!is.null(prior.prob) && length(prior.prob)!=lu) stop("universe and prior.prob must have same length")
+		if(!is.null(covariate) && length(covariate)!=lu) stop("universe and covariate must have same length")
+	}
 
 #	Fit trend in DE with respect to the covariate, combining all de lists
 	if(!is.null(covariate)) {
+		if(!is.null(prior.prob)) warning("prior.prob being recomputed from covariate")
 		covariate <- as.numeric(covariate)
-		if(length(covariate) != length(covariate)) stop("universe and covariate must have same length")
 		isDE <- as.numeric(universe %in% unlist(de))
 		o <- order(covariate)
 		prior.prob <- covariate
 		span <- approx(x=c(20,200),y=c(1,0.5),xout=sum(isDE),rule=2)$y
 		prior.prob[o] <- tricubeMovingAverage(isDE[o],span=span)
-		if(plot) barcodeplot(covariate, index=(isDE==1), worm=TRUE, span.worm=span)
+		if(plot) barcodeplot(covariate, index=as.logical(isDE), worm=TRUE, span.worm=span, main="DE status vs covariate")
 	}
 
-#	Enable REST-style online access to KEGG pathways
-	if(!requireNamespace("KEGGREST",quietly=TRUE)) stop("KEGGREST required but is not available")
+#	Get pathway annotation
+	if(is.null(gene.pathway))
+		EG.KEGG <- getGeneKEGGLinks(species.KEGG, convert=convert)
+	else {
+		EG.KEGG <- gene.pathway
+		d <- dim(EG.KEGG)
+		if(is.null(d)) stop("gene.pathway must be data.frame or matrix")
+		if(d[2] < 2) stop("gene.pathway must have at least 2 columns")
+		isna <- rowSums(is.na(EG.KEGG[,1:2])) > 0.5
+		EG.KEGG <- EG.KEGG[!isna,]
+	}
 
-#	Convert to KEGG organism/species codes
-	organism <- switch(species, "Hs"="hsa", "Dm"="dme", "Mm"="mmu", "Rn"="rno")
-
-#	Get gene-KEGG mappings, and remove duplicate entries
+#	Make sure that universe matches annotated genes
 	if(is.null(universe)) {
-
-		path <- KEGGREST::keggLink("pathway", organism)
-		path <- data.frame(kegg_id = names(path), path_id = path, stringsAsFactors = FALSE)
-		if(organism == "dme") {
-			EG <- KEGGREST::keggConv("dme", "ncbi-geneid")
-			geneid <- names(EG)[match(path$kegg_id, EG)]
-			geneid <- gsub("ncbi-geneid:", "", geneid)
-		} else {
-			geneid <- gsub(paste0(organism, ":"), "", path$kegg_id)
-		}
-		EG.KEGG <- data.frame(gene_id = geneid, path, stringsAsFactors = FALSE)
-
-		universe <- unique(EG.KEGG$gene_id)
-		universe <- as.character(universe)
+		universe <- as.character(unique(EG.KEGG[,1]))
 	} else {
 
+#		Consolidate replicated entries in universe, averaging corresponding prior.probs
 		universe <- as.character(universe)
-
 		dup <- duplicated(universe)
 		if(!is.null(prior.prob)) {
-			if(length(prior.prob)!=length(universe)) stop("length(prior.prob) must equal length(universe)")
 			prior.prob <- rowsum(prior.prob,group=universe,reorder=FALSE)
 			n <- rowsum(rep_len(1L,length(universe)),group=universe,reorder=FALSE)
 			prior.prob <- prior.prob/n
 		}
 		universe <- universe[!dup]
 
-		path <- KEGGREST::keggLink("pathway", organism)
-		path <- data.frame(kegg_id = names(path), path_id = path, stringsAsFactors = FALSE)
-		if(organism == "dme") {
-			EG <- KEGGREST::keggConv("dme", "ncbi-geneid")
-			geneid <- names(EG)[match(path$kegg_id, EG)]
-			geneid <- gsub("ncbi-geneid:", "", geneid)
-		} else {
-			geneid <- gsub(paste0(organism, ":"), "", path$kegg_id)
-		}
-		EG.KEGG <- data.frame(gene_id = geneid, path, stringsAsFactors = FALSE)
-
-		m <- match(EG.KEGG$gene_id, universe)
-		universe <- universe[m]
+#		Restrict universe to genes with annotation
+		m <- match(EG.KEGG[,1], universe)
+		InUni <- !is.na(m)
+		m <- m[InUni]
+		universe <- unique(universe[m])
 		if(!is.null(prior.prob)) prior.prob <- prior.prob[m]
-		EG.KEGG <- EG.KEGG[EG.KEGG$gene_id %in% universe,]
+
+#		Restrict annotation to genes in universe
+		EG.KEGG <- EG.KEGG[InUni,]
 	}
 
-	Total <- length(unique(EG.KEGG$gene_id))
-	if(Total<1L) stop("No genes found in universe")
+	NGenes <- length(universe)
+	if(NGenes<1L) stop("No annotated genes found in universe")
 
-#	Check prior probabilities
-	if(!is.null(prior.prob)) {
-		if(length(prior.prob)!=length(universe)) stop("length(prior.prob) must equal length(universe)")
+#	Restrict DE genes to universe
+	nde <- rep_len(0L,nsets)
+	for (s in 1:nsets) {
+		de[[s]] <- de[[s]][de[[s]] %in% universe]
+		nde[s] <- length(de[[s]])
 	}
 
-#	Overlap with DE genes
-	isDE <- lapply(de, function(x) EG.KEGG$gene_id %in% x)
-	TotalDE <- lapply(isDE, function(x) length(unique(EG.KEGG$gene_id[x])))
-	nDE <- length(isDE)
+#	Overlap pathways with DE genes
+	if(is.null(prior.prob)) {
+		X <- matrix(1,nrow(EG.KEGG),nsets+1)
+		colnames(X) <- c("N",names(de))
+	} else {
+		X <- matrix(1,nrow(EG.KEGG),nsets+2)
+		X[,nsets+2] <- prior.prob
+		colnames(X) <- c("N",names(de),"PP")
+	}
+	for (s in 1:nsets) X[,s+1] <- (EG.KEGG[,1] %in% de[[s]])
 
-	if(length(prior.prob)) {
-	#	Probability weight for each gene
-		m <- match(EG.KEGG$gene_id, universe)
-		PW2 <- list(prior.prob[m])
-		X <- do.call(cbind, c(N=1, isDE, PW=PW2))
-	} else
-		X <- do.call(cbind, c(N=1, isDE))
-
-	group <- EG.KEGG$path_id
-	S <- rowsum(X, group=group, reorder=FALSE)
-
-	P <- matrix(0, nrow = nrow(S), ncol = nsets)
+#	Count #genes and #DE genes and sum prior.prob for each pathway
+	S <- rowsum(X, group=EG.KEGG[,2], reorder=FALSE)
 
+#	Overlap tests
+	PValue <- matrix(0,nrow=nrow(S),ncol=nsets)
 	if(length(prior.prob)) {
 
 #		Calculate average prior prob for each set
-		PW.ALL <- sum(prior.prob[universe %in% EG.KEGG$gene_id])
-		AVE.PW <- S[,"PW"]/S[,"N"]
-		W <- AVE.PW*(Total-S[,"N"])/(PW.ALL-S[,"N"]*AVE.PW)
+		SumPP <- sum(prior.prob)
+		AvePP.set <- S[,"PP"]/S[,"N"]
+		W <- AvePP.set*(NGenes-S[,"N"])/(SumPP-S[,"N"]*AvePP.set)
 
 #		Wallenius' noncentral hypergeometric test
 		if(!requireNamespace("BiasedUrn",quietly=TRUE)) stop("BiasedUrn package required but is not available")
-		for(j in 1:nsets) for(i in 1:nrow(S)) 
-			P[i,j] <- BiasedUrn::pWNCHypergeo(S[i,1+j], S[i,"N"], Total-S[i,"N"], TotalDE[[j]], W[i],lower.tail=FALSE) + BiasedUrn::dWNCHypergeo(S[i,1+j], S[i,"N"], Total-S[i,"N"], TotalDE[[j]], W[i])
+		for(j in 1:nsets) for(i in 1:nrow(S))
+			PValue[i,j] <- BiasedUrn::pWNCHypergeo(S[i,1+j], S[i,"N"], NGenes-S[i,"N"], nde[j], W[i], lower.tail=FALSE) + BiasedUrn::dWNCHypergeo(S[i,1+j], S[i,"N"], NGenes-S[i,"N"], nde[j], W[i])
+
+#		Remove sum of prob column, not needed for output
 		S <- S[,-ncol(S)]
 
 	} else {
 
 #		Fisher's exact test
 		for(j in 1:nsets)
-			P[,j] <- phyper(q=S[,1+j]-0.5,m=TotalDE[[j]],n=Total-TotalDE[[j]], k=S[,"N"],lower.tail=FALSE)
+			PValue[,j] <- phyper(q=S[,1+j]-0.5,m=nde[j],n=NGenes-nde[j], k=S[,"N"],lower.tail=FALSE)
+
+	}
+
+#	Get list of pathway names
+	if(is.null(pathway.names))
+		PathID.PathName <- getKEGGPathwayNames(species.KEGG, remove.qualifier=TRUE)
+	else {
+		PathID.PathName <- pathway.names
+		d <- dim(PathID.PathName)
+		if(is.null(d)) stop("pathway.names must be data.frame or matrix")
+		if(d[2] < 2) stop("pathway.names must have at least 2 columns")
+		isna <- rowSums(is.na(PathID.PathName[,1:2])) > 0.5
+		PathID.PathName <- PathID.PathName[!isna,]
 
 	}
 
 #	Assemble output
 	g <- rownames(S)
-	pathname <- KEGGREST::keggList("pathway")
-	names(pathname) <- gsub("map", organism, names(pathname))
-	m <- match(g, names(pathname))
-	Results <- data.frame(Pathway = pathname[m], S, P, stringsAsFactors=FALSE)
+	m <- match(g, PathID.PathName[,1])
+	Results <- data.frame(Pathway = PathID.PathName[m,2], S, PValue, stringsAsFactors=FALSE)
 	rownames(Results) <- g
 
 #	Name p-value columns
@@ -225,10 +240,48 @@ kegga.default <- function(de, universe = NULL, species = "Hs", prior.prob = NULL
 	Results
 }
 
+getGeneKEGGLinks <- function(species.KEGG="hsa", convert=FALSE)
+#	Read pathway-gene mapping from KEGG website
+#	Gordon Smyth
+#	Created 7 Jan 2016.  Last modified 11 Feb 2016.
+{
+	URL <- paste("http://rest.kegg.jp/link/pathway",species.KEGG,sep="/")
+	Path.Gene <- read.table(URL,sep="\t",quote="\"",fill=TRUE,comment.char="",stringsAsFactors=FALSE)
+	colnames(Path.Gene) <- c("GeneID","PathwayID")
+
+#	Human, mouse, rat and chimp IDs are already Entrez, so don't need to convert
+	if(convert && species.KEGG %in% c("hsa","mmu","rno","ptr")) convert <- FALSE
+
+	if(convert) {
+#		Convert KEGG IDs to Entrez Gene IDs
+		URL <- paste("http://rest.kegg.jp/conv",species.KEGG,"ncbi-geneid",sep="/")
+		EntrezID.KEGGGeneID <- read.table(URL,sep="\t",quote="\"",fill=TRUE,comment.char="",stringsAsFactors=FALSE)
+		m <- match(Path.Gene[,1],EntrezID.KEGGGeneID[,2])
+		Path.Gene[,1] <- EntrezID.KEGGGeneID[m,1]
+		Path.Gene[,1] <- sub("^ncbi-geneid:", "", Path.Gene[,1])
+	} else {
+		Path.Gene[,1] <- sub(paste0("^",species.KEGG,":"), "", Path.Gene[,1])
+	}
+	Path.Gene
+}
+
+getKEGGPathwayNames <- function(species.KEGG=NULL, remove.qualifier=FALSE)
+#	Read pathways from KEGG website
+#	Gordon Smyth
+#	Created 7 Jan 2016.  Last modified 8 Jan 2016.
+{
+	URL <- "http://rest.kegg.jp/list/pathway"
+	if(!is.null(species.KEGG)) URL <- paste(URL,species.KEGG,sep="/")
+	PathID.PathName <- read.table(URL,sep="\t",quote="\"",fill=TRUE,comment.char="",stringsAsFactors=FALSE)
+	colnames(PathID.PathName) <- c("PathwayID","Description")
+	if(remove.qualifier) PathID.PathName[,2] <- removeExt(PathID.PathName[,2], sep=" - ")
+	PathID.PathName
+}
+
 topKEGG <- function(results, sort = NULL, number = 20L, truncate.path=NULL)
 #	Extract top KEGG pathways from kegga output 
 #	Gordon Smyth and Yifang Hu
-#	Created 15 May 2015. Modified 4 August 2015.
+#	Created 15 May 2015. Modified 25 Jan 2016.
 {
 #	Check results
 	if(!is.data.frame(results)) stop("results should be a data.frame.")
@@ -256,11 +309,11 @@ topKEGG <- function(results, sort = NULL, number = 20L, truncate.path=NULL)
 
 #	Sort by minimum p-value for specified gene lists
 	P.col <- 2L+nsets+isort
-	if(length(P.col)==1L) {
-		o <- order(results[,P.col])
-	} else {
-		o <- order(do.call("pmin",as.data.frame(results[,P.col,drop=FALSE])))
-	}
+	if(nsets==1L)
+		P <- results[,P.col]
+	else
+		P <- do.call(pmin,results[,P.col,drop=FALSE])
+	o <- order(P,results$N,results$Pathway)
 	tab <- results[o[1L:number],,drop=FALSE]
 
 #	Truncate Pathway column for readability
