@@ -3,7 +3,7 @@
 lmFit <- function(object,design=NULL,ndups=1,spacing=1,block=NULL,correlation,weights=NULL,method="ls",...)
 #	Fit genewise linear models
 #	Gordon Smyth
-#	30 June 2003.  Last modified 7 July 2015.
+#	30 June 2003.  Last modified 6 Oct 2015.
 {
 #	Extract components from y
 	y <- getEAWP(object)
@@ -51,9 +51,8 @@ lmFit <- function(object,design=NULL,ndups=1,spacing=1,block=NULL,correlation,we
 
 #	Possible warning on missing coefs
 	if(NCOL(fit$coef)>1) {
-		i <- is.na(fit$coef)
-		i <- apply(i[,1]==i[,-1,drop=FALSE],1,all)
-		n <- sum(!i)
+		n <- rowSums(is.na(fit$coef))
+		n <- sum(n>0 & n<NCOL(fit$coef))
 		if(n>0) warning("Partial NA coefficients for ",n," probe(s)",call.=FALSE) 
 	}
 
@@ -218,40 +217,51 @@ gls.series <- function(M,design=NULL,ndups=2,spacing=1,block=NULL,correlation=NU
 #	Fit linear model for each gene to a series of microarrays.
 #	Fit is by generalized least squares allowing for correlation between duplicate spots.
 #	Gordon Smyth
-#	11 May 2002.  Last revised 26 June 2015.
+#	11 May 2002.  Last revised 29 Dec 2015.
 {
+#	Check M
 	M <- as.matrix(M)
+	ngenes <- nrow(M)
 	narrays <- ncol(M)
+
+#	Check design
 	if(is.null(design)) design <- matrix(1,narrays,1)
 	design <- as.matrix(design)
 	if(nrow(design) != narrays) stop("Number of rows of design matrix does not match number of arrays")
+	nbeta <- ncol(design)
+	coef.names <- colnames(design)
+
+#	Check correlation
 	if(is.null(correlation)) correlation <- duplicateCorrelation(M,design=design,ndups=ndups,spacing=spacing,block=block,weights=weights,...)$consensus.correlation
+	if(abs(correlation) >= 1) stop("correlation is 1 or -1, so the model is degenerate")
+
+#	Check weights
 	if(!is.null(weights)) {
 		weights[is.na(weights)] <- 0
 		weights <- asMatrixWeights(weights,dim(M))
 		M[weights < 1e-15 ] <- NA
 		weights[weights < 1e-15] <- NA
 	}
-	nbeta <- ncol(design)
-	coef.names <- colnames(design)
+
+#	Unwrap duplicates (if necessary) and make correlation matrix
 	if(is.null(block)) {
+#		Correlated within-array probes
 		if(ndups<2) {
-			warning("No duplicates: correlation between duplicates set to zero")
+			warning("No duplicates (ndups<2)")
 			ndups <- 1
 			correlation <- 0
 		}
-		if(is.null(spacing)) spacing <- 1
 		cormatrix <- diag(rep(correlation,len=narrays),nrow=narrays,ncol=narrays) %x% array(1,c(ndups,ndups))
+		if(is.null(spacing)) spacing <- 1
 		M <- unwrapdups(M,ndups=ndups,spacing=spacing)
 		if(!is.null(weights)) weights <- unwrapdups(weights,ndups=ndups,spacing=spacing)
 		design <- design %x% rep(1,ndups)
 		colnames(design) <- coef.names
+		ngenes <- nrow(M)
+		narrays <- ncol(M)
 	} else {
-		if(ndups>1) {
-			stop("Cannot specify ndups>2 and non-null block argument")
-		} else {
-			ndups <- spacing <- 1
-		}
+#		Correlated samples
+		ndups <- spacing <- 1
 		block <- as.vector(block)
 		if(length(block)!=narrays) stop("Length of block does not match number of arrays")
 		ub <- unique(block)
@@ -260,9 +270,10 @@ gls.series <- function(M,design=NULL,ndups=2,spacing=1,block=NULL,correlation=NU
 		cormatrix <- Z%*%(correlation*t(Z))
 	}
 	diag(cormatrix) <- 1
-	ngenes <- nrow(M)
+
 	stdev.unscaled <- matrix(NA,ngenes,nbeta,dimnames=list(rownames(M),coef.names))
 
+#	If weights and missing values are absent, use a fast computation
 	NoProbeWts <- all(is.finite(M)) && (is.null(weights) || !is.null(attr(weights,"arrayweights")))
 	if(NoProbeWts) {
 		V <- cormatrix
@@ -300,6 +311,7 @@ gls.series <- function(M,design=NULL,ndups=2,spacing=1,block=NULL,correlation=NU
 		return(fit)
 	}
 
+#	Weights or missing values are present, to have to iterate over probes
 	beta <- stdev.unscaled
 	sigma <- rep(NA,ngenes)
 	df.residual <- rep(0,ngenes)
@@ -371,10 +383,10 @@ nonEstimable <- function(x)
 	}
 }
 
-fitted.MArrayLM <- function(object,design=object$design,...)
+fitted.MArrayLM <- function(object,...)
 #	Fitted values from MArray linear model fit
 #	Gordon Smyth
-#	29 November 2005
+#	29 November 2005.  Last modified 4 May 2016.
 {
 	object$coefficients %*% t(object$design)
 }
@@ -391,7 +403,7 @@ getEAWP <- function(object)
 #	Given any microarray data object, extract basic information needed
 #	for linear modelling.
 #	Gordon Smyth
-#	9 March 2008. Last modified 7 July 2015.
+#	9 March 2008. Last modified 18 July 2016.
 {
 	y <- list()
 	
@@ -401,7 +413,10 @@ getEAWP <- function(object)
 			y$exprs <- as.matrix(object$E)
 			y$Amean <- rowMeans(y$exprs,na.rm=TRUE)
 		} else {
+			if(is(object,"EListRaw")) stop("EListRaw object: please normalize first")
+			if(is(object,"RGList")) stop("RGList object: please normalize first")
 			y$printer <- object$printer
+			if(is.null(object$M)) stop("data object isn't of a recognized data class")
 			y$exprs <- as.matrix(object$M)
 			if(!is.null(object$A)) y$Amean <- rowMeans(as.matrix(object$A),na.rm=TRUE)
 		}
@@ -431,13 +446,20 @@ getEAWP <- function(object)
 		}
 		if(length(object@maA)) y$Amean <- rowMeans(object@maA,na.rm=TRUE)
 	} else {
+	if(is(object,"eSet")) {
+		if(!requireNamespace("Biobase",quietly=TRUE)) stop("Biobase package required but is not available")
+		y$exprs <- Biobase::assayDataElement(object,"exprs")
+		if(length(object@featureData@data)) y$probes <- object@featureData@data
+		y$Amean <- rowMeans(y$exprs,na.rm=TRUE)
+		if("weights" %in% Biobase::assayDataElementNames(object)) y$weights <- Biobase::assayDataElement(object,"weights")
+	} else {
 #		Default method for matrices, data.frames, vsn objects etc.
 		if(is.vector(object))
 			y$exprs <- matrix(object,nrow=1)
 		else
 			y$exprs <- as.matrix(object)
 		y$Amean <- rowMeans(y$exprs,na.rm=TRUE)
-	}}}}
+	}}}}}
 
 #	Check expression values are numeric
 	if(mode(y$exprs) != "numeric") stop("Data object doesn't contain numeric expression values")
